%CCFM_bif_diagram.m, by Daniel Lyons (March 2014):
%Purpose: Computes a bifurcation diagram (lambda vs. min/max(x1) for the
%CCFM system of 3 ODEs over the range: -3<lambda<3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; clc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%filepath (needs to be changed locally to folder containing "CCFM_manifolds")
mypath = '/Users/danielclyons/Dropbox/CCFM_large_folder';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath(strcat(mypath,'/CCFM_manifolds/functions/'));
addpath(strcat(mypath,'/CCFM_manifolds/Hopf_lambda_minus_files/'));
addpath(strcat(mypath,'/CCFM_manifolds/Hopf_lambda_plus_files/'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%fixed point continuation
%Performs fixed point continuation of the 27 fixed points of the CCFM
%device
global c lambda;
ode_system_fsolve = 'fluxgate'; %specify fluxgate system
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%solution tolerance (f_solve)
sol_tol = 1.0e-8;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%supply initial fixed point values at lambda = -0.43
fixed_points_array_0p43 = [
 
     0.769708796 0.999622906 -0.999316952; %saddle (2D stable)
     0.999622906 -0.999316952 0.769708796; %(construct 2D stable manifolds)
    -0.999316952 0.769708796 0.999622906;
    
    -0.769708796 -0.999622906 0.999316952;
    -0.999622906 0.999316952 -0.769708796;
      0.999316952 -0.769708796 -0.999622906;
 
 
    -0.970586289 -0.627477624 -0.887742722; % saddle (2D stable)
    -0.627477624 -0.887742722 -0.970586289;
    -0.887742722 -0.970586289 -0.627477624;

     0.970586289 0.627477624 0.887742722;
    0.627477624 0.887742722 0.970586289;
    0.887742722 0.970586289 0.627477624;
     
    0.497328593 0.733515654 0.98003978; %saddle (2D unstable)
    0.98003978 0.497328593 0.733515654;
    0.733515654 0.98003978 0.497328593;

    -0.497328593 -0.733515654 -0.98003978;
    -0.98003978 -0.497328593 -0.733515654;
    -0.733515654 -0.98003978 -0.497328593;
    
    -0.9994542667 0.8564065259 0.99962304 %async sinks (3D stable)
    0.99962304 -0.9994542667 0.8564065259
    0.8564065259 0.99962304 -0.9994542667

    0.9994542667 -0.8564065259 -0.99962304
    -0.99962304 0.9994542667 -0.8564065259
    -0.8564065259 -0.99962304 0.9994542667
    
     
      0.9166123443 0.9166123443 0.9166123443 %sync sinks (3D stable)
      -0.9166123443 -0.9166123443 -0.9166123443

      0 0 0 %3D source
];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 1
%initialize lambda array info
dir_cont = -1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  250;%750;
lambda_min = -0.45;
lambda_max = -0.43;
lambda_range = [lambda_min lambda_max];

%use initial fixed point values at lambda = -0.43
fixed_points_array = fixed_points_array_0p43;
%plot equilibrium branch color-coded for stability
final_fixed_point_array = ...
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 2 (a)
%initialize lambda array info
dir_cont = -1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  100;%750;
lambda_min = -0.50;
lambda_max = -0.45;
lambda_range = [lambda_min lambda_max];
%supply fixed_points at lambda = -0.45
fixed_points_array = [
      final_fixed_point_array(25,:) %sync sinks (3D stable)
      final_fixed_point_array(26,:)

      final_fixed_point_array(27,:) %3D source
];
%plot equilibrium branch color-coded for stability
final_fixed_point_array = ...
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol); 
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 2 (b)
%initialize lambda array info
dir_cont = -1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  100;%750;
lambda_min = -0.505;
lambda_max = -0.50;
lambda_range = [lambda_min lambda_max];
%supply fixed_points at lambda = -0.45
fixed_points_array = final_fixed_point_array;
%plot equilibrium branch color-coded for stability
final_fixed_point_array = ...
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol); 
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 2 (c)
%initialize lambda array info
dir_cont = -1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  100;%750;
lambda_min = -0.65;
lambda_max = -0.505;
lambda_range = [lambda_min lambda_max];
%supply fixed_points at lambda = -0.45
fixed_points_array = final_fixed_point_array;
%plot equilibrium branch color-coded for stability
final_fixed_point_array = ...
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol); 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 2 (d)
%initialize lambda array info
dir_cont = -1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  300;%750;
lambda_min = -0.6667;%-0.66666667;
lambda_max = -0.65;
lambda_range = [lambda_min lambda_max];
%supply fixed_points at lambda = -0.45
fixed_points_array = [
      final_fixed_point_array(1,:) %sync sinks (3D stable)
      final_fixed_point_array(2,:)

      final_fixed_point_array(3,:) %3D source
];
sol_tol_pfb = 1.0e-6;%fsolve solution tolerance
%plot equilibrium branch color-coded for stability
final_fixed_point_array = ...
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol_pfb); 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 3
%initialize lambda array info
dir_cont = -1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  100;%750;
lambda_min = -3.0;
lambda_max = -0.6667;%-0.66666667;
lambda_range = [lambda_min lambda_max];
%supply fixed_points at lambda = -0.45
fixed_points_array = [
      final_fixed_point_array(3,:); %3D source
];
%plot equilibrium branch color-coded for stability
final_fixed_point_array = ...
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 4
%initialize lambda array info
dir_cont = 1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  173;%750;
lambda_min = -0.43;
lambda_max = 0.43;
lambda_range = [lambda_min lambda_max];

%use initial fixed point values at lambda = -0.43
fixed_points_array = fixed_points_array_0p43;

%plot equilibrium branch color-coded for stability
final_fixed_point_array = ...
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 5
%initialize lambda array info
dir_cont = 1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  4000;%750;
lambda_min = 0.43;
lambda_max = 0.44;
lambda_range = [lambda_min lambda_max];

%use initial fixed point values at lambda = -0.43
fixed_points_array = final_fixed_point_array;

%plot equilibrium branch color-coded for stability
final_fixed_point_array = ...
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run 6
%initialize lambda array info
dir_cont = 1; %direction of continuation 1 (lambda_min to lambda_max) or -1 (lambda_max to lambda_min)
num_lambda =  257;%750;
lambda_min = 0.44;
lambda_max = 3.0;
lambda_range = [lambda_min lambda_max];

%use initial fixed point values at lambda = -0.43
fixed_points_array = final_fixed_point_array(25:27,:);

%plot equilibrium branch color-coded for stability
final_fixed_point_array = ...
    branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%generate stable limit cycle (via numerical integration)
lambda_max = -3.0;
delta_lambda = 0.04;%(lambda_max - lambda_min)/(num_lambda-1);
opts = odeset('abstol',1e-5,'reltol',1e-5);
lambda_array_lc = [];
x1_max_stable_limit_cycle = []; x1_min_stable_limit_cycle = [];
lambda = lambda_max;
while lambda < -0.2
    y0 = 5.0*((-1).^(0:2)).*randn(1,3);
    %numerical integration from 0 to 30 (remove transcience)
    [~,y] = ode23('odeccfm',[0:0.1:30],y0,opts);
    y0 = y(end,:);
    %numerical integration from 0 to 30
    [~,y] = ode23('odeccfm',[0:0.025:50],y0,opts);%numerical integration from 0 to 30
    
    %check for oscillations(1) or steady state(0)
    [osc,x1_max_amp,x1_min_amp] = osc_check(y(:,1));
    if osc==1
        lambda_array_lc = [lambda_array_lc lambda];
        x1_max_stable_limit_cycle = [x1_max_stable_limit_cycle x1_max_amp];
        %x1_min_stable_limit_cycle = [x1_min_stable_limit_cycle x1_min_amp];
    else
        break;%break out of loop
    end
    lambda = lambda + delta_lambda %reset lambda for next run
end
hold on;
plot(lambda_array_lc,x1_max_stable_limit_cycle,'k.','markersize',20);
plot(lambda_array_lc,-x1_max_stable_limit_cycle,'k.','markersize',20);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%label figure
set(gca,'fontsize',18);
xlabel('\lambda','fontsize',18);
ylabel('x_1','fontsize',18);
ylim([-1.1 1.1]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%add unstable limit cycles from Hopf (lambda=-0.5018)

mat=importdata('Hopf2.xlsx');%import data
string1 = 'mat.data.c';

% 
% t = data.data(:,1);
% x1 = data.data(:,2);
% x2 = data.data(:,3);
% x3 = data.data(:,4);
% data.textdata(2,1)

lambda_array = [-0.50175 -0.4995 -0.4882 -0.4681 -0.4519 -0.4475 ];
period_array = [18.11 18.65 22.05 34.64 85.82 469.2];
%reverse arrays
lambda_array = lambda_array(end:-1:1);
period_array = period_array(end:-1:1);

num_lambda = length(lambda_array);
ulc_n_max = zeros(num_lambda,1);

for ii=1:num_lambda

    %unstable limit cycle with negative lambda
    data_string = strcat(string1,num2str(num_lambda+1-ii)); %record the command as a string
    data = eval(data_string); %execute the command, saving the data
    t = data(:,1);
    x1 = data(:,2);
    x2 = data(:,3);
    x3 = data(:,4);
    
    ulc_n_max(ii) = max(x1);
end

hold on; plot(lambda_array,ulc_n_max,'ko','markersize',10);
hold on; plot(lambda_array,-ulc_n_max,'ko','markersize',10);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%add unstable limit cycle from Hopf (lambda=4/3)

mat=importdata('chopf.xlsx');%import data
string1 = 'mat.data.c';
lambda_array = [1.311 1.242 1.142 1.039 0.9335 0.8283 0.7559 0.6464...
   0.5415 0.4592];
period_array = [1.91 2.237 2.833 3.629 4.676 6.09 7.411 10.49 17.13 50.4];
 lambda_array = lambda_array(end:-1:1);
 period_array = period_array(end:-1:1);


num_lambda = length(lambda_array);
ulc_n_max = zeros(num_lambda,1);

for ii=1:num_lambda

    %unstable limit cycle with negative lambda
    data_string = strcat(string1,num2str(num_lambda+1-ii)); %record the command as a string
    data = eval(data_string); %execute the command, saving the data
    t = data(:,1);
    x1 = data(:,2);
    x2 = data(:,3);
    x3 = data(:,4);
    
    if ii==num_lambda-4
        data1 = importdata('curve.dat');
        data2 = importdata('curve2.dat');
        data3 = importdata('curve3.dat');
        t = data1(:,1);
        x1 = data1(:,2);
        x2 = data2(:,2);
        x3 = data3(:,2);
    end
    
    
    ulc_n_max(ii) = max(x1);
end

hold on; plot(lambda_array,ulc_n_max,'ko','markersize',10);
hold on; plot(lambda_array,-ulc_n_max,'ko','markersize',10);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
