%branch_plot.m, by Daniel Lyons (March 2014)
%Plots a branch in the lambda vs. x1 bifurcation diagram according to
%stability (number of negative real part eigenvalues of Jacobian)

function final_fixed_point_array = branch_plot(num_lambda,lambda_range,dir_cont,fixed_points_array,ode_system_fsolve,sol_tol)

%set parameters
global lambda c;
c = 3.0;



num_fp = size(fixed_points_array,1);

%initialize lambda array info
lambda_min = lambda_range(1);
lambda_max = lambda_range(2);
delta_lambda = (lambda_max - lambda_min)/(num_lambda-1);
%delta_lambda = 1;


%direction of continuation (dir_cont)
%forward (dir_cont=1):lambda_min to lambda_max
%backward (dir_cont=-1):lambda_max to lambda_min
if dir_cont == 1
    lambda_array = lambda_min:delta_lambda:lambda_max;
elseif dir_cont == -1
    lambda_array = lambda_max:-delta_lambda:lambda_min;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%preallocate arrays for saving fixed points
fp_array_cont = zeros(num_fp,3*num_lambda);
fp_array_cont(:,1:3) = fixed_points_array;

fp_array_type  = zeros(num_fp,num_lambda);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Perform continuation of fixed points
for ii=1:num_lambda
        lambda = lambda_array(ii);
        disp([' ']); disp([' ']);
        disp(['ii = ',num2str(ii),' out of ',num2str(num_lambda),', lambda = ',num2str(lambda)]);

    
    start_index = 3*ii-2;
    stop_index  = 3*ii;
    %fixed_points_array = fp_array_cont(:,start_index:stop_index);
    
    fixed_points_array = compute_fixed_points(ode_system_fsolve,fixed_points_array,sol_tol);
    fp_array_cont(:,start_index:stop_index) = fixed_points_array;
    
    %compute stability info for each fixed point
    for jj=1:num_fp
           x0 = fixed_points_array(jj,:);

        if isnan(x0(1))
            fp_array_type(jj,ii) = 0;
        else
            %compute Jacobian
            DF = CCFM_Jac(x0);
            %compute stability info
            [type,eig_val]  = stability_info(DF);
            %save point type if fixed point exists
            fp_array_type(jj,ii) = type;
        end
    end
    
    
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%plotting
%1('k-'): (sink: 3D stable manifold (3 eigenvalues with negative real part)
%2:('k--') (source: 3D unstable manifold (3 eigenvalues with positive real part)
%3:('b--') (saddle: 2D stable manifold (2 eigenvalues with negative real part)
%4:('r--') (saddle: 2D unstable manifold (2 eigenvalues with positive real part)
%color_hash={1,2,3,4}
color_hash = {'k-','k--','b--','r--'};

%jj=1;%fixed point
for jj = 1:num_fp
    %plot according to fixed point stability
    ii=1;%lambda value
    start_lambda_index = 1;
    while ii < num_lambda
        %search for change in fixed point type (bifurcation)
        while ((fp_array_type(jj,ii+1)-fp_array_type(jj,ii))==0)
            ii = ii+1;

            %check to see if the end of the array is reached on next iteration
            if (ii+1)==num_lambda
                ii = ii+1;
                break;%break out of inner while loop at end of array
            end
        end
        
        %plot by type if fixed point exists
        if fp_array_type(jj,start_lambda_index+1)~=0
            stop_lambda_index = ii;%ii-1;%ii;
            x1_index_start = 3*start_lambda_index-2;
            x1_index_stop  = 3*stop_lambda_index-2;
            plot_type = color_hash{fp_array_type(jj,start_lambda_index+1)};
            hold on;
            plot(lambda_array(start_lambda_index:stop_lambda_index),fp_array_cont(jj,x1_index_start:3:x1_index_stop),plot_type);
        end
        
        %reset index for next run
        start_lambda_index = ii;
        ii = ii + 1;
    end

end


%save final fixed points for next round of plotting
final_fixed_point_array = fp_array_cont(:,3*num_lambda-2:3*num_lambda);