%stability_info.m, by Daniel Lyons (March 2014)
%Given a jacobian matrix (Jac) corresponding to a fixed point, the
%eigenvalues of the matrix are computed and stored as the array eig_val,
%and the variable type is output as an integer (0 to 4) based on the
%stability information (the eigenvalues) at the fixed point. Type is chosen
%as follows:
%0: no information
%1: (sink: 3D stable manifold (3 eigenvalues with negative real part)
%2: (source: 3D unstable manifold (3 eigenvalues with positive real part)
%3: (saddle: 2D stable manifold (2 eigenvalues with negative real part)
%4: (saddle: 2D unstable manifold (2 eigenvalues with positive real part)

function  [type,eig_val]  = stability_info(Jac)

type = 0;%initialize

if sum(sum(isnan(Jac))) ~= 0
    eig_val = NaN*ones(3,3);
    return; %end function call with type=0
end
%compute eigenvalues
eig_val = eig(Jac);

%initialize
num_eig_val_pos = 0;
num_eig_val_neg = 0;

%tally number of eigenvalues with negative and positive real part
eig_val_real = real(eig_val);
for ii=1:3
    if eig_val_real(ii) > 0
        num_eig_val_pos = num_eig_val_pos + 1;
    elseif eig_val_real(ii) < 0
        num_eig_val_neg = num_eig_val_neg + 1;
    elseif eig_val_real(ii) == 0
    end
end

%classify type of point
if (num_eig_val_neg==3)&&(num_eig_val_pos==0)
    type = 1;
elseif (num_eig_val_neg==0)&&(num_eig_val_pos==3)
    type = 2;
elseif (num_eig_val_neg==2)&&(num_eig_val_pos==1)
    type = 3;
elseif (num_eig_val_neg==1)&&(num_eig_val_pos==2)
    type = 4;
end

